@@ -1,6 +1,6 @@
 """Extra array functions built on top of the array API standard."""
 
-from ._delegation import isclose, pad
+from ._delegation import isclose, one_hot, pad
 from ._lib._at import at
 from ._lib._funcs import (
     apply_where,
@@ -8,6 +8,7 @@ from ._lib._funcs import (
     broadcast_shapes,
     cov,
     create_diagonal,
+    default_dtype,
     expand_dims,
     kron,
     nunique,
@@ -16,7 +17,7 @@ from ._lib._funcs import (
 )
 from ._lib._lazy import lazy_apply
 
-__version__ = "0.7.2"
+__version__ = "0.8.0"
 
 # pylint: disable=duplicate-code
 __all__ = [
@@ -27,11 +28,13 @@ __all__ = [
     "broadcast_shapes",
     "cov",
     "create_diagonal",
+    "default_dtype",
     "expand_dims",
     "isclose",
     "kron",
     "lazy_apply",
     "nunique",
+    "one_hot",
     "pad",
     "setdiff1d",
     "sinc",
